{% load i18n %}

<div class="panel panel-default">
    <div class="panel-body">
        <div class="alert alert-warning">
            <p>{% translate "You either don't have access to this board, or this board doesn't exist." %}</p>
        </div>
    </div>
</div>
